@define-color month_border_color black;

.sources-button > button:hover,
.sources-button > button:active,
.sources-button > button:checked {
    box-shadow: inset 1px 0 @borders, inset -1px 0 @borders;
}
datechooser .weeknum {
    color: @theme_fg_color;
    background-color: alpha(@theme_fg_color, 0.05);
    border: 1px solid alpha(currentColor, 0.5);
}

